@@ -13,27 +13,26 @@
 #    under the License.
 
 import abc
+from collections import abc as collections_abc
 import datetime
-from distutils import versionpredicate
 import re
 import uuid
 import warnings
 
 import copy
-import iso8601
 import netaddr
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import six
+from oslo_utils import versionutils
 
-from oslo_versionedobjects._i18n import _, _LE
+from oslo_versionedobjects._i18n import _
 from oslo_versionedobjects import _utils
 from oslo_versionedobjects import exception
 
 
 class KeyTypeError(TypeError):
     def __init__(self, expected, value):
-        super(KeyTypeError, self).__init__(
+        super().__init__(
             _('Key %(key)s must be of type %(expected)s not %(actual)s'
               ) % {'key': repr(value),
                    'expected': expected.__name__,
@@ -43,7 +42,7 @@ class KeyTypeError(TypeError):
 
 class ElementTypeError(TypeError):
     def __init__(self, expected, key, value):
-        super(ElementTypeError, self).__init__(
+        super().__init__(
             _('Element %(key)s:%(val)s must be of type %(expected)s'
               ' not %(actual)s'
               ) % {'key': key,
@@ -53,8 +52,7 @@ class ElementTypeError(TypeError):
                    })
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AbstractFieldType(object):
+class AbstractFieldType(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def coerce(self, obj, attr, value):
         """This is called to coerce (if possible) a value on assignment.
@@ -131,11 +129,11 @@ class FieldType(AbstractFieldType):
         raise NotImplementedError()
 
 
-class UnspecifiedDefault(object):
+class UnspecifiedDefault:
     pass
 
 
-class Field(object):
+class Field:
     def __init__(self, field_type, nullable=False,
                  default=UnspecifiedDefault, read_only=False):
         self._type = field_type
@@ -144,8 +142,18 @@ class Field(object):
         self._read_only = read_only
 
     def __repr__(self):
-        return '%s(default=%s,nullable=%s)' % (self._type.__class__.__name__,
-                                               self._default, self._nullable)
+        if isinstance(self._default, set):
+            # TODO(stephenfin): Drop this when we switch from
+            # 'inspect.getargspec' to 'inspect.getfullargspec', since our
+            # hashes will have to change anyway
+            # make a py27 and py35 compatible representation. See bug 1771804
+            default = 'set([%s])' % ','.join(
+                sorted([str(v) for v in self._default])
+            )
+        else:
+            default = str(self._default)
+        return '{}(default={},nullable={})'.format(
+            self._type.__class__.__name__, default, self._nullable)
 
     @property
     def nullable(self):
@@ -255,14 +263,13 @@ class String(FieldType):
     @staticmethod
     def coerce(obj, attr, value):
         # FIXME(danms): We should really try to avoid the need to do this
-        accepted_types = six.integer_types + (float, six.string_types,
-                                              datetime.datetime)
+        accepted_types = (int, float, str, datetime.datetime)
         if isinstance(value, accepted_types):
-            return six.text_type(value)
-        else:
-            raise ValueError(_('A string is required in field %(attr)s, '
-                               'not a %(type)s') %
-                             {'attr': attr, 'type': type(value).__name__})
+            return str(value)
+
+        raise ValueError(_('A string is required in field %(attr)s, '
+                           'not a %(type)s') %
+                         {'attr': attr, 'type': type(value).__name__})
 
     @staticmethod
     def stringify(value):
@@ -278,15 +285,19 @@ class SensitiveString(String):
     Passwords in the string value are masked when stringified.
     """
     def stringify(self, value):
-        return super(SensitiveString, self).stringify(
+        return super().stringify(
             strutils.mask_password(value))
 
 
 class VersionPredicate(String):
     @staticmethod
     def coerce(obj, attr, value):
+        if not isinstance(value, str):
+            raise ValueError(_('Version %(val)s should be a string type, not '
+                               '%(real_type)s') %
+                             {'val': value, 'real_type': type(value)})
         try:
-            versionpredicate.VersionPredicate('check (%s)' % value)
+            versionutils.VersionPredicate(value)
         except ValueError:
             raise ValueError(_('Version %(val)s is not a valid predicate in '
                                'field %(attr)s') %
@@ -301,11 +312,11 @@ class Enum(String):
         try:
             # Test validity of the values
             for value in valid_values:
-                super(Enum, self).coerce(None, 'init', value)
+                super().coerce(None, 'init', value)
         except (TypeError, ValueError):
             raise exception.EnumValidValuesInvalidError()
         self._valid_values = valid_values
-        super(Enum, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     @property
     def valid_values(self):
@@ -315,30 +326,44 @@ class Enum(String):
         if value not in self._valid_values:
             msg = _("Field value %s is invalid") % value
             raise ValueError(msg)
-        return super(Enum, self).coerce(obj, attr, value)
+        return super().coerce(obj, attr, value)
 
     def stringify(self, value):
         if value not in self._valid_values:
             msg = _("Field value %s is invalid") % value
             raise ValueError(msg)
-        return super(Enum, self).stringify(value)
+        return super().stringify(value)
 
     def get_schema(self):
-        return {'enum': self._valid_values}
+        schema = super().get_schema()
+        schema['enum'] = self._valid_values
+        return schema
 
 
-class UUID(FieldType):
+class StringPattern(FieldType):
+    def get_schema(self):
+        if hasattr(self, "PATTERN"):
+            return {'type': ['string'], 'pattern': self.PATTERN}
+        else:
+            msg = _("%s has no pattern") % self.__class__.__name__
+            raise AttributeError(msg)
 
-    _PATTERN = (r'^[a-fA-F0-9]{8}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]'
-                r'{4}-?[a-fA-F0-9]{12}$')
+
+class UUID(StringPattern):
+
+    PATTERN = (r'^[a-fA-F0-9]{8}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]{4}-?[a-fA-F0-9]'
+               r'{4}-?[a-fA-F0-9]{12}$')
 
     @staticmethod
     def coerce(obj, attr, value):
         # FIXME(danms): We should actually verify the UUIDness here
         with warnings.catch_warnings():
-            warnings.simplefilter("once")
+            # Change the warning action only if no other filter exists
+            # for this warning to allow the client to define other action
+            # like 'error' for this warning.
+            warnings.filterwarnings(action="once", append=True)
             try:
-                uuid.UUID(str(value))
+                uuid.UUID("%s" % value)
             except Exception:
                 # This is to ensure no breaking behaviour for current
                 # users
@@ -348,47 +373,40 @@ class UUID(FieldType):
                               "release. Please update your "
                               "code to input valid UUIDs or accept "
                               "ValueErrors for invalid UUIDs. See "
-                              "http://docs.openstack.org/developer/oslo.versionedobjects/api/fields.html#oslo_versionedobjects.fields.UUIDField "  # noqa
-                              "for further details" % value, FutureWarning)
+                              "https://docs.openstack.org/oslo.versionedobjects/latest/reference/fields.html#oslo_versionedobjects.fields.UUIDField "  # noqa
+                              "for further details" %
+                              repr(value).encode('utf8'),
+                              FutureWarning)
 
-            return str(value)
-
-    def get_schema(self):
-        return {'type': ['string'], 'pattern': self._PATTERN}
+            return "%s" % value
 
 
-class MACAddress(FieldType):
+class MACAddress(StringPattern):
 
-    _PATTERN = r'^[0-9a-f]{2}(:[0-9a-f]{2}){5}$'
-    _REGEX = re.compile(_PATTERN)
+    PATTERN = r'^[0-9a-f]{2}(:[0-9a-f]{2}){5}$'
+    _REGEX = re.compile(PATTERN)
 
     @staticmethod
     def coerce(obj, attr, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             lowered = value.lower().replace('-', ':')
             if MACAddress._REGEX.match(lowered):
                 return lowered
-        raise ValueError(_LE("Malformed MAC %s"), value)
-
-    def get_schema(self):
-        return {'type': ['string'], 'pattern': self._PATTERN}
+        raise ValueError(_("Malformed MAC %s") % (value,))
 
 
-class PCIAddress(FieldType):
+class PCIAddress(StringPattern):
 
-    _PATTERN = r'^[0-9a-f]{4}:[0-9a-f]{2}:[0-1][0-9a-f].[0-7]$'
-    _REGEX = re.compile(_PATTERN)
+    PATTERN = r'^[0-9a-f]{4}:[0-9a-f]{2}:[0-1][0-9a-f].[0-7]$'
+    _REGEX = re.compile(PATTERN)
 
     @staticmethod
     def coerce(obj, attr, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             newvalue = value.lower()
             if PCIAddress._REGEX.match(newvalue):
                 return newvalue
-        raise ValueError(_LE("Malformed PCI address %s"), value)
-
-    def get_schema(self):
-        return {'type': ['string'], 'pattern': self._PATTERN}
+        raise ValueError(_("Malformed PCI address %s") % (value,))
 
 
 class Integer(FieldType):
@@ -408,6 +426,9 @@ class NonNegativeInteger(FieldType):
             raise ValueError(_('Value must be >= 0 for field %s') % attr)
         return v
 
+    def get_schema(self):
+        return {'type': ['integer'], 'minimum': 0}
+
 
 class Float(FieldType):
     def coerce(self, obj, attr, value):
@@ -425,6 +446,9 @@ class NonNegativeFloat(FieldType):
             raise ValueError(_('Value must be >= 0 for field %s') % attr)
         return v
 
+    def get_schema(self):
+        return {'type': ['number'], 'minimum': 0}
+
 
 class Boolean(FieldType):
     @staticmethod
@@ -444,10 +468,10 @@ class FlexibleBoolean(Boolean):
 class DateTime(FieldType):
     def __init__(self, tzinfo_aware=True, *args, **kwargs):
         self.tzinfo_aware = tzinfo_aware
-        super(DateTime, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def coerce(self, obj, attr, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             # NOTE(danms): Being tolerant of isotime strings here will help us
             # during our objects transition
             value = timeutils.parse_isotime(value)
@@ -460,7 +484,7 @@ class DateTime(FieldType):
             # NOTE(danms): Legacy objects from sqlalchemy are stored in UTC,
             # but are returned without a timezone attached.
             # As a transitional aid, assume a tz-naive object is in UTC.
-            value = value.replace(tzinfo=iso8601.iso8601.Utc())
+            value = value.replace(tzinfo=datetime.timezone.utc)
         elif not self.tzinfo_aware:
             value = value.replace(tzinfo=None)
         return value
@@ -480,13 +504,13 @@ class DateTime(FieldType):
         return _utils.isotime(value)
 
 
-class IPAddress(FieldType):
+class IPAddress(StringPattern):
     @staticmethod
     def coerce(obj, attr, value):
         try:
             return netaddr.IPAddress(value)
         except netaddr.AddrFormatError as e:
-            raise ValueError(six.text_type(e))
+            raise ValueError(str(e))
 
     def from_primitive(self, obj, attr, value):
         return self.coerce(obj, attr, value)
@@ -535,7 +559,8 @@ class IPV4AndV6Address(IPAddress):
         return result
 
     def get_schema(self):
-        return {'type': ['string'], 'format': 'ipv6'}
+        return {'oneOf': [IPV4Address().get_schema(),
+                          IPV6Address().get_schema()]}
 
 
 class IPNetwork(IPAddress):
@@ -544,38 +569,35 @@ class IPNetwork(IPAddress):
         try:
             return netaddr.IPNetwork(value)
         except netaddr.AddrFormatError as e:
-            raise ValueError(six.text_type(e))
+            raise ValueError(str(e))
 
 
 class IPV4Network(IPNetwork):
 
-    _PATTERN = (r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-'
-                r'9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/([0-9]|[1-2]['
-                r'0-9]|3[0-2]))$')
+    PATTERN = (r'^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-'
+               r'9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/([0-9]|[1-2]['
+               r'0-9]|3[0-2]))$')
 
     @staticmethod
     def coerce(obj, attr, value):
         try:
             return netaddr.IPNetwork(value, version=4)
         except netaddr.AddrFormatError as e:
-            raise ValueError(six.text_type(e))
-
-    def get_schema(self):
-        return {'type': ['string'], 'pattern': self._PATTERN}
+            raise ValueError(str(e))
 
 
 class IPV6Network(IPNetwork):
 
     def __init__(self, *args, **kwargs):
-        super(IPV6Network, self).__init__(*args, **kwargs)
-        self._PATTERN = self._create_pattern()
+        super().__init__(*args, **kwargs)
+        self.PATTERN = self._create_pattern()
 
     @staticmethod
     def coerce(obj, attr, value):
         try:
             return netaddr.IPNetwork(value, version=6)
         except netaddr.AddrFormatError as e:
-            raise ValueError(six.text_type(e))
+            raise ValueError(str(e))
 
     def _create_pattern(self):
         ipv6seg = '[0-9a-fA-F]{1,4}'
@@ -607,19 +629,16 @@ class IPV6Network(IPNetwork):
             'fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|'
             # ::255.255.255.255 ::ffff:255.255.255.255 ::ffff:0:255.255.255.255
             '::(ffff(:0{1,4}){0,1}:){0,1}'
-            '(' + ipv4seg + '\.){3,3}' +
+            '(' + ipv4seg + r'\.){3,3}' +
             ipv4seg + '|'
             # 2001:db8:3:4::192.0.2.33 64:ff9b::192.0.2.33
             '(' + ipv6seg + ':){1,4}:'
-            '(' + ipv4seg + '\.){3,3}' +
+            '(' + ipv4seg + r'\.){3,3}' +
             ipv4seg +
             # /128
-            '(\/(d|dd|1[0-1]d|12[0-8]))$'
+            r'(\/(d|dd|1[0-1]d|12[0-8]))$'
             )
 
-    def get_schema(self):
-        return {'type': ['string'], 'pattern': self._PATTERN}
-
 
 class CompoundFieldType(FieldType):
     def __init__(self, element_type, **field_args):
@@ -628,7 +647,9 @@ class CompoundFieldType(FieldType):
 
 class List(CompoundFieldType):
     def coerce(self, obj, attr, value):
-        if not isinstance(value, list):
+
+        if (not isinstance(value, collections_abc.Iterable) or
+                isinstance(value, (str, collections_abc.Mapping))):
             raise ValueError(_('A list is required in field %(attr)s, '
                                'not a %(type)s') %
                              {'attr': attr, 'type': type(value).__name__})
@@ -666,26 +687,27 @@ class Dict(CompoundFieldType):
         primitive = {}
         for key, element in value.items():
             primitive[key] = self._element_type.to_primitive(
-                obj, '%s["%s"]' % (attr, key), element)
+                obj, '{}["{}"]'.format(attr, key), element)
         return primitive
 
     def from_primitive(self, obj, attr, value):
         concrete = {}
         for key, element in value.items():
             concrete[key] = self._element_type.from_primitive(
-                obj, '%s["%s"]' % (attr, key), element)
+                obj, '{}["{}"]'.format(attr, key), element)
         return concrete
 
     def stringify(self, value):
         return '{%s}' % (
-            ','.join(['%s=%s' % (key, self._element_type.stringify(val))
+            ','.join(['{}={}'.format(key, self._element_type.stringify(val))
                       for key, val in sorted(value.items())]))
 
     def get_schema(self):
-        return {'type': ['object']}
+        return {'type': ['object'],
+                'additionalProperties': self._element_type.get_schema()}
 
 
-class DictProxyField(object):
+class DictProxyField:
     """Descriptor allowing us to assign pinning data as a dict of key_types
 
     This allows us to have an object field that will be a dict of key_type
@@ -694,7 +716,7 @@ class DictProxyField(object):
     This will take care of the conversion while the dict field will make sure
     that we store the raw json-serializable data on the object.
 
-    key_type should return a type that unambiguously responds to six.text_type
+    key_type should return a type that unambiguously responds to str
     so that calling key_type on it yields the same thing.
     """
     def __init__(self, dict_field_name, key_type=int):
@@ -707,14 +729,13 @@ class DictProxyField(object):
         if getattr(obj, self._fld_name) is None:
             return
         return {self._key_type(k): v
-                for k, v in six.iteritems(getattr(obj, self._fld_name))}
+                for k, v in getattr(obj, self._fld_name).items()}
 
     def __set__(self, obj, val):
         if val is None:
             setattr(obj, self._fld_name, val)
         else:
-            setattr(obj, self._fld_name,
-                    {six.text_type(k): v for k, v in six.iteritems(val)})
+            setattr(obj, self._fld_name, {str(k): v for k, v in val.items()})
 
 
 class Set(CompoundFieldType):
@@ -733,8 +754,8 @@ class Set(CompoundFieldType):
             self._element_type.to_primitive(obj, attr, x) for x in value)
 
     def from_primitive(self, obj, attr, value):
-        return set([self._element_type.from_primitive(obj, attr, x)
-                    for x in value])
+        return {self._element_type.from_primitive(obj, attr, x)
+                for x in value}
 
     def stringify(self, value):
         return 'set([%s])' % (
@@ -749,7 +770,7 @@ class Object(FieldType):
     def __init__(self, obj_name, subclasses=False, **kwargs):
         self._obj_name = obj_name
         self._subclasses = subclasses
-        super(Object, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     @staticmethod
     def _get_all_obj_names(obj):
@@ -817,14 +838,62 @@ class Object(FieldType):
         else:
             ident = ''
 
-        return '%s%s' % (self._obj_name, ident)
+        return '{}{}'.format(value.obj_name(), ident)
+
+    def get_schema(self):
+        from oslo_versionedobjects import base as obj_base
+        obj_classes = obj_base.VersionedObjectRegistry.obj_classes()
+        if self._obj_name in obj_classes:
+            cls = obj_classes[self._obj_name][0]
+            namespace_key = cls._obj_primitive_key('namespace')
+            name_key = cls._obj_primitive_key('name')
+            version_key = cls._obj_primitive_key('version')
+            data_key = cls._obj_primitive_key('data')
+            changes_key = cls._obj_primitive_key('changes')
+            field_schemas = {key: field.get_schema()
+                             for key, field in cls.fields.items()}
+            required_fields = [key for key, field in sorted(cls.fields.items())
+                               if not field.nullable]
+            schema = {
+                'type': ['object'],
+                'properties': {
+                    namespace_key: {
+                        'type': 'string'
+                    },
+                    name_key: {
+                        'type': 'string'
+                    },
+                    version_key: {
+                        'type': 'string'
+                    },
+                    changes_key: {
+                        'type': 'array',
+                        'items': {
+                            'type': 'string'
+                        }
+                    },
+                    data_key: {
+                        'type': 'object',
+                        'description': 'fields of %s' % self._obj_name,
+                        'properties': field_schemas,
+                    },
+                },
+                'required': [namespace_key, name_key, version_key, data_key]
+            }
+
+            if required_fields:
+                schema['properties'][data_key]['required'] = required_fields
+
+            return schema
+        else:
+            raise exception.UnsupportedObjectError(objtype=self._obj_name)
 
 
 class AutoTypedField(Field):
     AUTO_TYPE = None
 
     def __init__(self, **kwargs):
-        super(AutoTypedField, self).__init__(self.AUTO_TYPE, **kwargs)
+        super().__init__(self.AUTO_TYPE, **kwargs)
 
 
 class StringField(AutoTypedField):
@@ -858,7 +927,7 @@ class BaseEnumField(AutoTypedField):
                 typename=self.AUTO_TYPE.__class__.__name__,
                 fieldname=self.__class__.__name__)
 
-        super(BaseEnumField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     def __repr__(self):
         valid_values = self._type.valid_values
@@ -867,9 +936,9 @@ class BaseEnumField(AutoTypedField):
             'default': self._default,
             }
         args.update({'valid_values': valid_values})
-        return '%s(%s)' % (self._type.__class__.__name__,
-                           ','.join(['%s=%s' % (k, v)
-                                     for k, v in sorted(args.items())]))
+        return '{}({})'.format(self._type.__class__.__name__,
+                               ','.join(['{}={}'.format(k, v)
+                                         for k, v in sorted(args.items())]))
 
     @property
     def valid_values(self):
@@ -882,14 +951,14 @@ class EnumField(BaseEnumField):
 
     This class allows for anonymous enum types to be
     declared, simply by passing in a list of valid values
-    to its constructor. It is generally preferrable though,
+    to its constructor. It is generally preferable though,
     to create an explicit named enum type by sub-classing
     the BaseEnumField type directly.
     '''
 
     def __init__(self, valid_values, **kwargs):
         self.AUTO_TYPE = Enum(valid_values=valid_values)
-        super(EnumField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 class StateMachine(EnumField):
@@ -943,7 +1012,7 @@ class StateMachine(EnumField):
         return 'unknown'
 
     def coerce(self, obj, attr, value):
-        super(StateMachine, self).coerce(obj, attr, value)
+        super().coerce(obj, attr, value)
         my_name = self._my_name(obj)
         msg = _("%(object)s.%(name)s is not allowed to transition out of "
                 "%(value)s state")
@@ -984,7 +1053,7 @@ class UUIDField(AutoTypedField):
     your own objects/fields.py
 
     :Example:
-         .. code-block:: python
+        .. code-block:: python
 
             import oslo_versionedobjects.fields as ovo_fields
 
@@ -997,8 +1066,8 @@ class UUIDField(AutoTypedField):
             class UUIDField(ovo_fields.AutoTypedField):
                 AUTO_TYPE = UUID()
 
-        and then in your objects use
-        ``<your_projects>.object.fields.UUIDField``.
+    and then in your objects use
+    ``<your_projects>.object.fields.UUIDField``.
 
     This will become default behaviour in the future.
     """
@@ -1047,7 +1116,7 @@ class FlexibleBooleanField(AutoTypedField):
 class DateTimeField(AutoTypedField):
     def __init__(self, tzinfo_aware=True, **kwargs):
         self.AUTO_TYPE = DateTime(tzinfo_aware=tzinfo_aware)
-        super(DateTimeField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 class DictOfStringsField(AutoTypedField):
@@ -1073,7 +1142,7 @@ class DictOfListOfStringsField(AutoTypedField):
 class ListOfEnumField(AutoTypedField):
     def __init__(self, valid_values, **kwargs):
         self.AUTO_TYPE = List(Enum(valid_values))
-        super(ListOfEnumField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     def __repr__(self):
         valid_values = self._type._element_type._type.valid_values
@@ -1082,9 +1151,9 @@ class ListOfEnumField(AutoTypedField):
             'default': self._default,
             }
         args.update({'valid_values': valid_values})
-        return '%s(%s)' % (self._type.__class__.__name__,
-                           ','.join(['%s=%s' % (k, v)
-                                     for k, v in sorted(args.items())]))
+        return '{}({})'.format(self._type.__class__.__name__,
+                               ','.join(['{}={}'.format(k, v)
+                                         for k, v in sorted(args.items())]))
 
 
 class SetOfIntegersField(AutoTypedField):
@@ -1107,14 +1176,18 @@ class ObjectField(AutoTypedField):
     def __init__(self, objtype, subclasses=False, **kwargs):
         self.AUTO_TYPE = Object(objtype, subclasses)
         self.objname = objtype
-        super(ObjectField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
 
 class ListOfObjectsField(AutoTypedField):
     def __init__(self, objtype, subclasses=False, **kwargs):
         self.AUTO_TYPE = List(Object(objtype, subclasses))
         self.objname = objtype
-        super(ListOfObjectsField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
+
+
+class ListOfUUIDField(AutoTypedField):
+    AUTO_TYPE = List(UUID())
 
 
 class IPAddressField(AutoTypedField):
@@ -1145,12 +1218,12 @@ class IPV6NetworkField(AutoTypedField):
     AUTO_TYPE = IPV6Network()
 
 
-class CoercedCollectionMixin(object):
+class CoercedCollectionMixin:
     def __init__(self, *args, **kwargs):
         self._element_type = None
         self._obj = None
         self._field = None
-        super(CoercedCollectionMixin, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def enable_coercing(self, element_type, obj, field):
         self._element_type = element_type
@@ -1177,32 +1250,30 @@ class CoercedList(CoercedCollectionMixin, list):
             step = i.step or 1
             coerced_items = [self._coerce_item(start + index * step, item)
                              for index, item in enumerate(y)]
-            super(CoercedList, self).__setitem__(i, coerced_items)
+            super().__setitem__(i, coerced_items)
         else:
-            super(CoercedList, self).__setitem__(i, self._coerce_item(i, y))
+            super().__setitem__(i, self._coerce_item(i, y))
 
     def append(self, x):
-        super(CoercedList, self).append(self._coerce_item(len(self) + 1, x))
+        super().append(self._coerce_item(len(self) + 1, x))
 
     def extend(self, t):
-        l = len(self)
-        coerced_items = [self._coerce_item(l + index, item)
+        coerced_items = [self._coerce_item(len(self) + index, item)
                          for index, item in enumerate(t)]
-        super(CoercedList, self).extend(coerced_items)
+        super().extend(coerced_items)
 
     def insert(self, i, x):
-        super(CoercedList, self).insert(i, self._coerce_item(i, x))
+        super().insert(i, self._coerce_item(i, x))
 
     def __iadd__(self, y):
-        l = len(self)
-        coerced_items = [self._coerce_item(l + index, item)
+        coerced_items = [self._coerce_item(len(self) + index, item)
                          for index, item in enumerate(y)]
-        return super(CoercedList, self).__iadd__(coerced_items)
+        return super().__iadd__(coerced_items)
 
     def __setslice__(self, i, j, y):
         coerced_items = [self._coerce_item(i + index, item)
                          for index, item in enumerate(y)]
-        return super(CoercedList, self).__setslice__(i, j, coerced_items)
+        return super().__setslice__(i, j, coerced_items)
 
 
 class CoercedDict(CoercedCollectionMixin, dict):
@@ -1214,38 +1285,34 @@ class CoercedDict(CoercedCollectionMixin, dict):
 
     def _coerce_dict(self, d):
         res = {}
-        for key, element in six.iteritems(d):
+        for key, element in d.items():
             res[key] = self._coerce_item(key, element)
         return res
 
     def _coerce_item(self, key, item):
-        if not isinstance(key, six.string_types):
-            # NOTE(guohliu) In order to keep compatibility with python3
-            # we need to use six.string_types rather than basestring here,
-            # since six.string_types is a tuple, so we need to pass the
-            # real type in.
-            raise KeyTypeError(six.string_types[0], key)
+        if not isinstance(key, str):
+            raise KeyTypeError(str, key)
         if hasattr(self, "_element_type") and self._element_type is not None:
-            att_name = "%s[%s]" % (self._field, key)
+            att_name = "{}[{}]".format(self._field, key)
             return self._element_type.coerce(self._obj, att_name, item)
         else:
             return item
 
     def __setitem__(self, key, value):
-        super(CoercedDict, self).__setitem__(key,
-                                             self._coerce_item(key, value))
+        super().__setitem__(key,
+                            self._coerce_item(key, value))
 
     def update(self, other=None, **kwargs):
         if other is not None:
-            super(CoercedDict, self).update(self._coerce_dict(other),
-                                            **self._coerce_dict(kwargs))
+            super().update(self._coerce_dict(other),
+                           **self._coerce_dict(kwargs))
         else:
-            super(CoercedDict, self).update(**self._coerce_dict(kwargs))
+            super().update(**self._coerce_dict(kwargs))
 
     def setdefault(self, key, default=None):
-        return super(CoercedDict, self).setdefault(key,
-                                                   self._coerce_item(key,
-                                                                     default))
+        return super().setdefault(key,
+                                  self._coerce_item(key,
+                                                    default))
 
 
 class CoercedSet(CoercedCollectionMixin, set):
@@ -1257,7 +1324,8 @@ class CoercedSet(CoercedCollectionMixin, set):
     def _coerce_element(self, element):
         if hasattr(self, "_element_type") and self._element_type is not None:
             return self._element_type.coerce(self._obj,
-                                             "%s[%s]" % (self._field, element),
+                                             "{}[{}]".format(
+                                                 self._field, element),
                                              element)
         else:
             return element
@@ -1269,17 +1337,17 @@ class CoercedSet(CoercedCollectionMixin, set):
         return coerced
 
     def add(self, value):
-        return super(CoercedSet, self).add(self._coerce_element(value))
+        return super().add(self._coerce_element(value))
 
     def update(self, values):
-        return super(CoercedSet, self).update(self._coerce_iterable(values))
+        return super().update(self._coerce_iterable(values))
 
     def symmetric_difference_update(self, values):
-        return super(CoercedSet, self).symmetric_difference_update(
+        return super().symmetric_difference_update(
             self._coerce_iterable(values))
 
     def __ior__(self, y):
-        return super(CoercedSet, self).__ior__(self._coerce_iterable(y))
+        return super().__ior__(self._coerce_iterable(y))
 
     def __ixor__(self, y):
-        return super(CoercedSet, self).__ixor__(self._coerce_iterable(y))
+        return super().__ixor__(self._coerce_iterable(y))
