@@ -12,15 +12,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
 import copy
 import datetime
 import hashlib
 import inspect
 from unittest import mock
 
-import iso8601
-
 from oslo_versionedobjects import base
 from oslo_versionedobjects import exception
 from oslo_versionedobjects import fields
@@ -190,7 +187,7 @@ class TestObjectComparators(test.TestCase):
     def test_compare_obj_with_dt(self):
         mock_test = mock.Mock()
         mock_test.assertEqual = mock.Mock()
-        dt = datetime.datetime(1955, 11, 5, tzinfo=iso8601.iso8601.UTC)
+        dt = datetime.datetime(1955, 11, 5, tzinfo=datetime.timezone.utc)
         replaced_dt = dt.replace(tzinfo=None)
         my_obj = self.MyComparedObjectWithTZ(tzfield=dt)
         my_db_obj = {'tzfield': replaced_dt}
@@ -212,7 +209,7 @@ class FakeResource(base.VersionedObject):
 
 class TestObjectVersionChecker(test.TestCase):
     def setUp(self):
-        super(TestObjectVersionChecker, self).setUp()
+        super().setUp()
         objects = [MyObject, MyObject2, ]
         self.obj_classes = {obj.__name__: [obj] for obj in objects}
         self.ovc = fixture.ObjectVersionChecker(obj_classes=self.obj_classes)
@@ -538,9 +535,14 @@ class TestObjectVersionChecker(test.TestCase):
         exp_methods = sorted([('remotable_method', argspec),
                               ('remotable_classmethod', argspec)])
         expected_relevant_data = (exp_fields, exp_methods)
+        # NOTE(hberaud) the following hashlib usage will emit a bandit
+        # warning. It can be solved by passing `usedforsecurity=False` to
+        # the md5 function, however, this parameter was introduced with py39
+        # so passing it will break py38 unittest. I'd suggest to ignore this
+        # bandit rule while py38 is in our supported runtimes.
         expected_hash = hashlib.md5(bytes(repr(
-            expected_relevant_data).encode())).hexdigest()
-        expected_fp = '%s-%s' % (MyObject.VERSION, expected_hash)
+            expected_relevant_data).encode())).hexdigest()  # nosec
+        expected_fp = '{}-{}'.format(MyObject.VERSION, expected_hash)
 
         self.assertEqual(expected_fp, fp, "_get_fingerprint() did not "
                                           "generate a correct fingerprint.")
@@ -560,13 +562,19 @@ class TestObjectVersionChecker(test.TestCase):
         exp_fields = sorted(list(MyObject.fields.items()))
         exp_methods = sorted([('remotable_method', argspec),
                               ('remotable_classmethod', argspec)])
-        exp_child_versions = collections.OrderedDict(sorted(
-            child_versions.items()))
+        exp_child_versions = fixture.OsloOrderedDict(
+            sorted(child_versions.items())
+        )
         exp_relevant_data = (exp_fields, exp_methods, exp_child_versions)
 
+        # NOTE(hberaud) the following hashlib usage will emit a bandit
+        # warning. It can be solved by passing `usedforsecurity=False` to
+        # the md5 function, however, this parameter was introduced with py39
+        # so passing it will break py38 unittest. I'd suggest to ignore this
+        # bandit rule while py38 is in our supported runtimes.
         expected_hash = hashlib.md5(bytes(repr(
-            exp_relevant_data).encode())).hexdigest()
-        expected_fp = '%s-%s' % (MyObject.VERSION, expected_hash)
+            exp_relevant_data).encode())).hexdigest()  # nosec
+        expected_fp = '{}-{}'.format(MyObject.VERSION, expected_hash)
 
         self.assertEqual(expected_fp, fp, "_get_fingerprint() did not "
                                           "generate a correct fingerprint.")
@@ -594,9 +602,14 @@ class TestObjectVersionChecker(test.TestCase):
         exp_extra_data = ExtraDataObj
         exp_relevant_data = (exp_fields, exp_methods, exp_extra_data)
 
+        # NOTE(hberaud) the following hashlib usage will emit a bandit
+        # warning. It can be solved by passing `usedforsecurity=False` to
+        # the md5 function, however, this parameter was introduced with py39
+        # so passing it will break py38 unittest. I'd suggest to ignore this
+        # bandit rule while py38 is in our supported runtimes.
         expected_hash = hashlib.md5(bytes(repr(
-            exp_relevant_data).encode())).hexdigest()
-        expected_fp = '%s-%s' % (ExtraDataObj.VERSION, expected_hash)
+            exp_relevant_data).encode())).hexdigest()  # nosec
+        expected_fp = '{}-{}'.format(ExtraDataObj.VERSION, expected_hash)
 
         self.assertEqual(expected_fp, fp, "_get_fingerprint() did not "
                                           "generate a correct fingerprint.")
@@ -741,7 +754,7 @@ class TestStableObjectJsonFixture(test.TestCase):
 
 class TestMethodSpec(test.TestCase):
     def setUp(self):
-        super(TestMethodSpec, self).setUp()
+        super().setUp()
 
         def test_method1(a, b, kw1=123, **kwargs):
             pass
